@@ -64,9 +64,11 @@ MapView::MapView(Map *map, POI *poi, QWidget *parent) : QGraphicsView(parent)
 	_inputProjection = GCS::gcs(4326);
 	_hidpi = true;
 	_hillShading = true;
-	_layers = Layer::Raster | Layer::Vector;
+	_style = -1;
+	_layer = -1;
 	_map = map;
-	_map->load(_inputProjection, _outputProjection, _deviceRatio, _hidpi);
+	_map->load(_inputProjection, _outputProjection, _deviceRatio, _hidpi,
+	  _hillShading, _style, _layer);
 	connect(_map, &Map::tilesLoaded, this, &MapView::reloadMap);
 
 	_poi = poi;
@@ -441,9 +443,14 @@ void MapView::setMap(Map *map)
 
 	disconnect(_map, &Map::tilesLoaded, this, &MapView::reloadMap);
 	_map->unload();
+	if (map != _map) {
+		_style = -1;
+		_layer = -1;
+	}
 
 	_map = map;
-	_map->load(_inputProjection, _outputProjection, _deviceRatio, _hidpi);
+	_map->load(_inputProjection, _outputProjection, _deviceRatio, _hidpi,
+	  _hillShading, _style, _layer);
 	connect(_map, &Map::tilesLoaded, this, &MapView::reloadMap);
 
 	digitalZoom(0);
@@ -1165,12 +1172,6 @@ void MapView::drawBackground(QPainter *painter, const QRectF &rect)
 			flags = Map::Block;
 		else if (_opengl)
 			flags = Map::OpenGL;
-		if (_hillShading)
-			flags |= Map::HillShading;
-		if (_layers & Layer::Raster)
-			flags |= Map::Rasters;
-		if (_layers & Layer::Vector)
-			flags |= Map::Vectors;
 
 		_map->draw(painter, ir, flags);
 	}
@@ -1309,14 +1310,34 @@ void MapView::drawHillShading(bool draw)
 {
 	_hillShading = draw;
 
-	setMap(_map);
+	_map->unload();
+	_map->load(_inputProjection, _outputProjection, _deviceRatio, _hidpi,
+	  _hillShading, _style, _layer);
+
+	reloadMap();
 }
 
-void MapView::selectLayers(MapView::Layers layers)
+void MapView::selectStyle(int style)
 {
-	_layers = layers;
+	_style = style;
+	_layer = -1;
 
-	setMap(_map);
+	_map->unload();
+	_map->load(_inputProjection, _outputProjection, _deviceRatio, _hidpi,
+	  _hillShading, _style, _layer);
+
+	reloadMap();
+}
+
+void MapView::selectLayer(int layer)
+{
+	_layer = layer;
+
+	_map->unload();
+	_map->load(_inputProjection, _outputProjection, _deviceRatio, _hidpi,
+	  _hillShading, _style, _layer);
+
+	reloadMap();
 }
 
 void MapView::useStyles(bool use)
@@ -1481,12 +1502,15 @@ void MapView::drawInfoBackground(bool draw)
 
 void MapView::setHidpi(bool hidpi)
 {
+	_hidpi = hidpi;
+
 	QRectF vr(mapToScene(viewport()->rect()).boundingRect()
 	  .intersected(_map->bounds()));
 	RectC cr(_map->xy2ll(vr.topLeft()), _map->xy2ll(vr.bottomRight()));
 
 	_map->unload();
-	_map->load(_inputProjection, _outputProjection, _deviceRatio, hidpi);
+	_map->load(_inputProjection, _outputProjection, _deviceRatio, _hidpi,
+	  _hillShading, _style, _layer);
 	rescale();
 
 	QPointF nc = QRectF(_map->ll2xy(cr.topLeft()),
