@@ -7,9 +7,9 @@
 #include <QFileInfo>
 #include <QDir>
 #include <QXmlStreamReader>
+#include "common/util.h"
 #include "pcs.h"
 #include "utm.h"
-#include "metatype.h"
 #include "oruxmap.h"
 
 
@@ -486,10 +486,11 @@ int OruxMap::zoomOut()
 }
 
 void OruxMap::load(const Projection &in, const Projection &out,
-  qreal deviceRatio, bool hidpi, int style, int layer)
+  qreal deviceRatio, bool hidpi, bool hillShading, int style, int layer)
 {
 	Q_UNUSED(in);
 	Q_UNUSED(out);
+	Q_UNUSED(hillShading);
 	Q_UNUSED(style);
 	Q_UNUSED(layer);
 
